==> 1.0.0.tar.gz: lib/src/sub_types/FunctionBody.dart <==
import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../main/Expression.dart';
import '../main/Block.dart';

/*
<functionBody> ::= async? '=>' <expression> ';' (MODIFiED IT TO MAKE SEMICOLON OPTIONAL FOR: "invocation(() => e);")
| (async | async'*' | sync'*')? <block>

See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
Page 17
*/

class FunctionBody extends ParserType {
  Expression expression;
  Block block;
  bool hasIdentifier;
  bool hasSemicolon;
  int asyncType;

  FunctionBody(Lexer lexer, [bool requireSemiColon=true]) {
    // print ('Started FunctionBody');

    int snapShot = lexer.currentIndex;
    hasIdentifier = false;
    try {
      if (isType(lexer, TType.KEYWORD) && isVal(lexer, 'async')) {
        hasIdentifier = true;
        asyncType = 0;
        lexer.advance();
      }

      checkType(lexer, TType.EQUALS);
      lexer.advance();
      checkType(lexer, TType.RIGHT_ANGLE);
      lexer.advance();

      expression = Expression(lexer);

      if (requireSemiColon) {
        checkType(lexer, TType.SEMICOLON);
        lexer.advance();
        hasSemicolon = true;
      }
      else {
        checkNotType(lexer, TType.SEMICOLON);
        hasSemicolon = false;
        /*
        if (isType(lexer, TType.SEMICOLON)) {
          hasSemicolon = true;
          lexer.advance();
        }
        else {
          hasSemicolon = false;
        }
        */
      }
    }
    on WrongTokenType {
      lexer.setPos(snapShot);

      hasIdentifier = false;
      int attempt, maxAttempts = 3;
      snapShot = lexer.currentIndex;
      for (attempt = 0; attempt < maxAttempts; ++attempt) {
        try {
          switch (attempt) {
            case 0:
              checkType(lexer, TType.KEYWORD);
              checkVal(lexer, 'sync');
              lexer.advance();
              checkType(lexer, TType.MULTIPLY);
              lexer.advance();
              hasIdentifier = true;
              asyncType = 2;
              break;
            case 1:
              checkType(lexer, TType.KEYWORD);
              checkVal(lexer, 'async');
              lexer.advance();
              checkType(lexer, TType.MULTIPLY);
              lexer.advance();
              hasIdentifier = true;
              asyncType = 1;
              break;
            case 2:
              checkType(lexer, TType.KEYWORD);
              checkVal(lexer, 'async');
              lexer.advance();
              hasIdentifier = true;
              asyncType = 0;
              break;
          }
          
          break;
        }
        on WrongTokenType {
          lexer.setPos(snapShot);
        }
      }

      block = Block(lexer);
    }

    // print ('Finished FunctionBody');
  }

  String toString() {
    if (expression != null) {
      return (hasIdentifier ? 'async ' : '') + '=> $expression' + (hasSemicolon ? ';' : '');
    }
    else if (block != null) {
      String rtn = '';
      if (hasIdentifier) {
        switch (asyncType) {
          case 0: rtn += 'async '; break;
          case 1: rtn += 'async* '; break;
          case 2: rtn += 'sync* '; break;
        }
      }
      return rtn + '$block';
    }
    return '';
  }
  
}

==> 1.0.5.tar.gz: lib/src/sub_types/FunctionBody.dart <==
import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../main/Expression.dart';
import '../main/Block.dart';

/// <functionBody> ::= async? '=>' <expression> ';' (MODIFiED IT TO MAKE SEMICOLON OPTIONAL FOR: "invocation(() => e);")
/// | (async | async'*' | sync'*')? <block>
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 17

class FunctionBody extends ParserType {
  Expression expression;
  Block block;
  bool hasIdentifier;
  bool hasSemicolon;
  int asyncType;

  FunctionBody(Lexer lexer, [bool requireSemiColon = true]) {
    // print ('Started FunctionBody');

    int snapShot = lexer.currentIndex;
    hasIdentifier = false;
    try {
      if (isType(lexer, TType.KEYWORD) && isVal(lexer, 'async')) {
        hasIdentifier = true;
        asyncType = 0;
        lexer.advance();
      }

      checkType(lexer, TType.EQUALS);
      lexer.advance();
      checkType(lexer, TType.RIGHT_ANGLE);
      lexer.advance();

      expression = Expression(lexer);

      if (requireSemiColon) {
        checkType(lexer, TType.SEMICOLON);
        lexer.advance();
        hasSemicolon = true;
      } else {
        checkNotType(lexer, TType.SEMICOLON);
        hasSemicolon = false;
        /// if (isType(lexer, TType.SEMICOLON)) {
        /// hasSemicolon = true;
        /// lexer.advance();
        /// }
        /// else {
        /// hasSemicolon = false;
        /// }
      }
    } on WrongTokenType {
      lexer.setPos(snapShot);

      hasIdentifier = false;
      int attempt, maxAttempts = 3;
      snapShot = lexer.currentIndex;
      for (attempt = 0; attempt < maxAttempts; ++attempt) {
        try {
          switch (attempt) {
            case 0:
              checkType(lexer, TType.KEYWORD);
              checkVal(lexer, 'sync');
              lexer.advance();
              checkType(lexer, TType.MULTIPLY);
              lexer.advance();
              hasIdentifier = true;
              asyncType = 2;
              break;
            case 1:
              checkType(lexer, TType.KEYWORD);
              checkVal(lexer, 'async');
              lexer.advance();
              checkType(lexer, TType.MULTIPLY);
              lexer.advance();
              hasIdentifier = true;
              asyncType = 1;
              break;
            case 2:
              checkType(lexer, TType.KEYWORD);
              checkVal(lexer, 'async');
              lexer.advance();
              hasIdentifier = true;
              asyncType = 0;
              break;
          }

          break;
        } on WrongTokenType {
          lexer.setPos(snapShot);
        }
      }

      block = Block(lexer);
    }

    // print ('Finished FunctionBody');
  }

  String toString() {
    if (expression != null) {
      return (hasIdentifier ? 'async ' : '') +
          '=> $expression' +
          (hasSemicolon ? ';' : '');
    } else if (block != null) {
      String rtn = '';
      if (hasIdentifier) {
        switch (asyncType) {
          case 0:
            rtn += 'async ';
            break;
          case 1:
            rtn += 'async* ';
            break;
          case 2:
            rtn += 'sync* ';
            break;
        }
      }
      return rtn + '$block';
    }
    return '';
  }
}
